@@ -1,15 +1,18 @@
 import wx
 from typing import TYPE_CHECKING, Tuple, Union, Type
-from OpenGL.GL import (
-    glClear,
-    GL_DEPTH_BUFFER_BIT,
-)
+import logging
 import math
 import numpy
 import weakref
 
+from OpenGL.GL import (
+    glClear,
+    GL_DEPTH_BUFFER_BIT,
+)
+
 from amulet.api.data_types import PointCoordinates
 from amulet.api.level import BaseLevel
+from amulet.api.level.base_level.clone import PasteRule
 from amulet.api.structure import structure_cache
 from amulet.operations.paste import paste_iter
 from amulet.utils.matrix import (
@@ -46,6 +49,8 @@ from amulet_map_editor.programs.edit.api.key_config import ACT_BOX_CLICK
 if TYPE_CHECKING:
     from amulet_map_editor.programs.edit.api.canvas import EditCanvas
 
+log = logging.getLogger(__name__)
+
 BottomLeftRight = wx.BOTTOM | wx.LEFT | wx.RIGHT
 BottomLeftRightCentre = BottomLeftRight | wx.ALIGN_CENTER_HORIZONTAL
 BottomLeftRightExpand = BottomLeftRight | wx.EXPAND
@@ -466,6 +471,21 @@ class PasteTool(wx.BoxSizer, DefaultBaseToolUI):
             lang.get("program_3d_edit.paste_tool.copy_lava_tooltip")
         )
 
+        self._paste_rule = wx.Choice(
+            self._paste_panel,
+            choices=[
+                lang.get("program_3d_edit.paste_tool.paste_all"),
+                lang.get("program_3d_edit.paste_tool.paste_existing"),
+                lang.get("program_3d_edit.paste_tool.paste_not_existing"),
+            ],
+        )
+        self._paste_rule.SetSelection(0)
+        self._paste_sizer.Add(
+            self._paste_rule,
+            flag=BottomLeftRightExpand,
+            border=5,
+        )
+
         add_line()
 
         confirm_button = wx.Button(self._paste_panel, label="Confirm")
@@ -658,6 +678,10 @@ class PasteTool(wx.BoxSizer, DefaultBaseToolUI):
             level_index: int = fake_levels.active_level_index
             if level_index is not None:
                 render_level: RenderLevel = fake_levels.render_levels[level_index]
+                paste_rule = {
+                    1: PasteRule.PasteExist,
+                    2: PasteRule.PasteNotExist,
+                }.get(self._paste_rule.GetSelection(), PasteRule.PasteAll)
                 yield from paste_iter(
                     self.canvas.world,
                     self.canvas.dimension,
@@ -669,6 +693,7 @@ class PasteTool(wx.BoxSizer, DefaultBaseToolUI):
                     self._copy_air.GetValue(),
                     self._copy_water.GetValue(),
                     self._copy_lava.GetValue(),
+                    paste_rule,
                 )
         else:
             raise OperationSuccessful(
@@ -679,11 +704,14 @@ class PasteTool(wx.BoxSizer, DefaultBaseToolUI):
         self.canvas.run_operation(self._paste_operation)
 
     def _on_draw(self, evt):
-        self.canvas.renderer.start_draw()
-        if self.canvas.camera.projection_mode == Projection.PERSPECTIVE:
-            self.canvas.renderer.draw_sky_box()
-            glClear(GL_DEPTH_BUFFER_BIT)
-        self.canvas.renderer.draw_level()
-        self.canvas.renderer.draw_fake_levels()
-        self._selection.draw()
-        self.canvas.renderer.end_draw()
+        try:
+            self.canvas.renderer.start_draw()
+            if self.canvas.camera.projection_mode == Projection.PERSPECTIVE:
+                self.canvas.renderer.draw_sky_box()
+                glClear(GL_DEPTH_BUFFER_BIT)
+            self.canvas.renderer.draw_level()
+            self.canvas.renderer.draw_fake_levels()
+            self._selection.draw()
+            self.canvas.renderer.end_draw()
+        except Exception as e:
+            log.exception(f"Failed painting: {e}")
